@@ -191,38 +191,38 @@ setValidity2("DataFrame", .valid.DataFrame)
 ### like 'DataFrame(lapply(x, I))'. Not super convenient or intuitive!
 new_DataFrame <- function(listData=list(), nrows=NA, what="arguments")
 {
-        stopifnot(is.list(listData))
-        stopifnot(isSingleNumberOrNA(nrows))
-        if (!is.integer(nrows))
-            nrows <- as.integer(nrows)
-        listData_nrow <- nrow(listData)
-        if (is.null(listData_nrow)) {
-            ## 'listData' is NOT a data.frame or data-frame-like object.
-            if (length(listData) == 0L) {
-                if (is.na(nrows))
-                    nrows <- 0L
-                names(listData) <- character(0)
-            } else {
-                if (is.na(nrows)) {
-                    elt_nrows <- elementNROWS(listData)
-                    nrows <- elt_nrows[[1L]]
-                    if (!all(elt_nrows == nrows))
-                        stop(wmsg(what, " imply differing number of rows"))
-                }
-                if (is.null(names(listData)))
-                    names(listData) <- paste0("V", seq_along(listData))
-            }
+    stopifnot(is.list(listData))
+    stopifnot(isSingleNumberOrNA(nrows))
+    if (!is.integer(nrows))
+        nrows <- as.integer(nrows)
+    listData_nrow <- nrow(listData)
+    if (is.null(listData_nrow)) {
+        ## 'listData' is NOT a data.frame or data-frame-like object.
+        if (length(listData) == 0L) {
+            if (is.na(nrows))
+                nrows <- 0L
+            names(listData) <- character(0)
         } else {
-            ## 'listData' is a data.frame or data-frame-like object.
             if (is.na(nrows)) {
-                nrows <- listData_nrow
-            } else if (nrows != listData_nrow) {
-                stop(wmsg("the supplied 'nrows' does not match ",
-                          "the nb of rows in 'listData'"))
+                elt_nrows <- elementNROWS(listData)
+                nrows <- elt_nrows[[1L]]
+                if (!all(elt_nrows == nrows))
+                    stop(wmsg(what, " imply differing number of rows"))
             }
-            listData <- as.list(listData)
+            if (is.null(names(listData)))
+                names(listData) <- paste0("V", seq_along(listData))
         }
-        new2("DFrame", nrows=nrows, listData=listData, check=FALSE)
+    } else {
+        ## 'listData' is a data.frame or data-frame-like object.
+        if (is.na(nrows)) {
+            nrows <- listData_nrow
+        } else if (nrows != listData_nrow) {
+            stop(wmsg("the supplied 'nrows' does not match ",
+                      "the nb of rows in 'listData'"))
+        }
+        listData <- as.list(listData)
+    }
+    new2("DFrame", nrows=nrows, listData=listData, check=FALSE)
 }
 
 DataFrame <- function(..., row.names = NULL, check.names = TRUE,
@@ -309,6 +309,24 @@ DataFrame <- function(..., row.names = NULL, check.names = TRUE,
   ans
 }
 
+### Exported. Intended for developers to use in other packages and typically
+### not needed by the end user.
+### 3x faster than new("DFrame", nrows=nrow).
+### 500x faster than DataFrame(matrix(nrow=nrow, ncol=0L)).
+make_zero_col_DFrame <- function(nrow=0L)
+{
+    stopifnot(isSingleNumber(nrow))
+    if (!is.integer(nrow))
+        nrow <- as.integer(nrow)
+    stopifnot(nrow >= 0L)
+    new2("DFrame", nrows=nrow, check=FALSE)
+}
+
+### Alias for backward compatibility.
+### NOT exported but used in packages IRanges, GenomicRanges,
+### SummarizedExperiment, GenomicAlignments, and maybe more...
+make_zero_col_DataFrame <- make_zero_col_DFrame
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Subsetting
@@ -509,6 +527,9 @@ setMethod("mergeROWS", c("DataFrame", "ANY"),
     x
 }
 
+### Not a real default replaceCOLS() method for DataFrame objects (it actually
+### assumes that 'x' derives from SimpleList i.e. that 'x' is a DFrame object
+### or derivative).
 setMethod("replaceCOLS", c("DataFrame", "ANY"), function(x, i, value) {
     stopifnot(is.null(value) || is(value, "DataFrame"))
     sl <- as(x, "SimpleList")
@@ -520,7 +541,12 @@ setMethod("replaceCOLS", c("DataFrame", "ANY"), function(x, i, value) {
     sl <- .fill_short_columns(sl, max_len)
     names(sl) <- .make_colnames(sl, i, length(x), value)
     ri <- seq_len(max_len)
-    initialize(x, sl, rownames=.make_rownames(x, ri, ri, value), nrows=max_len)
+    ## Assumes that 'x' has a "listData" slot i.e. that 'x' is a DFrame object
+    ## or derivative.
+    BiocGenerics:::replaceSlots(x, listData=sl@listData,
+                                   elementMetadata=sl@elementMetadata,
+                                   rownames=.make_rownames(x, ri, ri, value),
+                                   nrows=max_len)
 })
 
 setMethod("normalizeSingleBracketReplacementValue", "DataFrame",
@@ -722,7 +748,12 @@ setAs("xtabs", "DFrame",
 }
 
 setAs("ANY", "DFrame", .defaultAsDataFrame)
+
 setAs("ANY", "DataFrame", function(from) as(from, "DFrame"))
+setAs("SimpleList", "DataFrame", function(from) as(from, "DFrame"))
+
+## Only temporarily needed (until we make DataFrame VIRTUAL).
+setAs("DFrame", "DataFrame", function(from) from)
 
 .VectorAsDataFrame <- function(from) {
   ans <- .defaultAsDataFrame(from)
@@ -827,6 +858,40 @@ setMethod("showAsCell", "DataFrame", showAsCell_array)
 ### Combining
 ###
 
+.format_mismatch_message <- function(x, y) {
+    universe <- union(x, y)
+    tx <- table(factor(x, levels=universe))
+    ty <- table(factor(y, levels=universe))
+    common <- pmin(tx, ty)
+
+    leftx <- tx - common
+    lefty <- ty - common
+    misleft <- names(leftx)[leftx > 0]
+    misright <- names(lefty)[lefty > 0]
+
+    .format_failed_colnames <- function(x) {
+        x <- sprintf("'%s'", x)
+        if (length(x)>3) x <- c(head(x, 3), "...")
+        paste(x, collapse=", ")
+    }
+
+    if (length(misleft) && length(misright)) {
+        msg <- paste0("(", .format_failed_colnames(misleft),
+            " vs ", .format_failed_colnames(misright), ")")
+    } else if (length(misleft)) {
+        msg <- paste0("(", .format_failed_colnames(misleft), " ",
+            if (length(misleft) > 1) "are" else "is",
+            " unique)")
+    } else {
+        msg <- paste0("(", .format_failed_colnames(misright), " ",
+            if (length(misright) > 1) "are" else "is",
+            " unique)")
+    }
+
+    stop(wmsg("the DataFrame objects to rbind do not have ",
+              "the same column names ", msg))
+}
+
 ### Return an integer matrix with 1 column per object in 'objects' and 1 row
 ### per column in 'x'.
 .make_colmaps <- function(x, objects)
@@ -834,14 +899,14 @@ setMethod("showAsCell", "DataFrame", showAsCell_array)
     x_colnames <- colnames(x)
     x_ncol <- length(x_colnames)
     map_x_colnames_to_object_colnames <- function(object_colnames) {
-        if (length(object_colnames) != x_ncol)
-            stop(wmsg("the DataFrame objects to rbind ",
-                      "must have the same number of columns"))
+        if (length(object_colnames) != x_ncol) {
+            .format_mismatch_message(x_colnames, object_colnames)
+        }
         colmap <- selectHits(findMatches(x_colnames, object_colnames),
                              select="first", nodup=TRUE)
-        if (anyNA(colmap))
-            stop(wmsg("the DataFrame objects to rbind ",
-                      "must have the same colnames"))
+        if (anyNA(colmap)) {
+            .format_mismatch_message(x_colnames, object_colnames)
+        }
         colmap
     }
     if (length(objects) == 0L) {
@@ -897,8 +962,8 @@ setMethod("showAsCell", "DataFrame", showAsCell_array)
     bindROWS(cols[[1L]], cols[-1L])
 }
 
-### Argument 'ignore.mcols' is ignored.
-.rbind_DataFrame_objects <-
+### Ignore the 'ignore.mcols' argument!
+.bindROWS_DataFrame_objects <-
     function(x, objects=list(), use.names=TRUE, ignore.mcols=FALSE, check=TRUE)
 {
     if (!isTRUEorFALSE(use.names))
@@ -927,7 +992,14 @@ setMethod("showAsCell", "DataFrame", showAsCell_array)
                  x_col <- x[[i]]
                  other_cols <- lapply(seq_along(objects),
                                       function(j) objects[[j]][[colmaps[i, j]]])
-                 .bind_cols_along_their_ROWS(c(list(x_col), other_cols))
+                 tryCatch(
+                     .bind_cols_along_their_ROWS(c(list(x_col), other_cols)),
+                     error=function(err) {
+                        stop("failed to rbind column '", colnames(x)[i],
+                            "' across DataFrame objects:\n  ",
+                            conditionMessage(err))
+                     }
+                 )
             }
         )
         ans_nrow <- NROW(ans_listData[[1L]])
@@ -952,22 +1024,15 @@ setMethod("showAsCell", "DataFrame", showAsCell_array)
                                    check=check)
 }
 
-setMethod("bindROWS", "DataFrame", .rbind_DataFrame_objects)
+### Defining bindROWS() gives us rbind().
+setMethod("bindROWS", "DataFrame", .bindROWS_DataFrame_objects)
 
-### The "rbind" method for DataFrame objects is just a thin wrapper around
-### a call to bindROWS().
-### Argument 'deparse.level' is ignored.
-rbind.DataFrame <- function(..., deparse.level=1)
-{
-    objects <- list(...)
-    bindROWS(objects[[1L]], objects[-1L], check=FALSE)
-}
-
-setMethod("rbind", "DataFrame", rbind.DataFrame)
-
-### Argument 'deparse.level' is ignored.
+### S3/S4 combo for cbind.DataFrame
 cbind.DataFrame <- function(..., deparse.level=1)
 {
+    if (!identical(deparse.level, 1))
+        warning(wmsg("the cbind() method for DataFrame objects ",
+                     "ignores the 'deparse.level' argument"))
     ## It's important that the call to DataFrame() below is able to deparse
     ## the arguments in ... so for example
     ##   b <- 11:13
@@ -982,7 +1047,6 @@ cbind.DataFrame <- function(..., deparse.level=1)
     ## with a column named "11:13".
     DataFrame(..., check.names=FALSE)
 }
-
 setMethod("cbind", "DataFrame", cbind.DataFrame)
 
 ### If we didn't define this method, calling c() on DataFrame objects would
