@@ -58,7 +58,7 @@ tail.DataTable <- tail.matrix
 setMethod("tail", "DataTable", tail.DataTable)
 
 setMethod("subset", "DataTable",
-          function(x, subset, select, drop = FALSE, ...) 
+          function(x, subset, select, drop = FALSE, ...)
           {
               i <- evalqForSubset(subset, x, ...)
               j <- evalqForSelect(select, x, ...)
@@ -133,29 +133,27 @@ setMethod("transform", "DataTable", transform.DataTable)
 ### Combining.
 ###
 
-setMethod("cbind", "DataTable", function(..., deparse.level=1)
-          stop("missing 'cbind' method for DataTable class ",
-               class(list(...)[[1L]])))
+### S3/S4 combo for rbind.DataTable
+rbind.DataTable <- function(..., deparse.level=1)
+{
+    if (!identical(deparse.level, 1))
+        warning(wmsg("the rbind() method for DataTable objects ",
+                     "ignores the 'deparse.level' argument"))
+    objects <- list(...)
+    bindROWS(objects[[1L]], objects=objects[-1L])
+}
+setMethod("rbind", "DataTable", rbind.DataTable)
 
-setMethod("rbind", "DataTable", function(..., deparse.level=1)
-          stop("missing 'rbind' method for DataTable class ",
-               class(list(...)[[1L]])))
-
-setMethod("cbind2", c("ANY", "DataTable"), function(x, y, ...) {
-  x <- as(x, "DataFrame")
-  cbind(x, y, ...)
-})
-
-setMethod("cbind2", c("DataTable", "ANY"), function(x, y, ...) {
-  y <- as(y, "DataFrame")
-  cbind(x, y, ...)
-})
-
-setMethod("cbind2", c("DataTable", "DataTable"), function(x, y, ...) {
-  x <- as(x, "DataFrame")
-  y <- as(y, "DataFrame")
-  cbind(x, y, ...)
-})
+### S3/S4 combo for cbind.DataTable
+cbind.DataTable <- function(..., deparse.level=1)
+{
+    if (!identical(deparse.level, 1))
+        warning(wmsg("the cbind() method for DataTable objects ",
+                     "ignores the 'deparse.level' argument"))
+    objects <- list(...)
+    bindCOLS(objects[[1L]], objects=objects[-1L])
+}
+setMethod("cbind", "DataTable", cbind.DataTable)
 
 setMethod("rbind2", c("ANY", "DataTable"), function(x, y, ...) {
   x <- as(x, "DataFrame")
@@ -173,13 +171,29 @@ setMethod("rbind2", c("DataTable", "DataTable"), function(x, y, ...) {
   rbind(x, y, ...)
 })
 
-.mergeByHits <- function(x, y, by, all.x=FALSE, all.y=FALSE, sort = TRUE, 
+setMethod("cbind2", c("ANY", "DataTable"), function(x, y, ...) {
+  x <- as(x, "DataFrame")
+  cbind(x, y, ...)
+})
+
+setMethod("cbind2", c("DataTable", "ANY"), function(x, y, ...) {
+  y <- as(y, "DataFrame")
+  cbind(x, y, ...)
+})
+
+setMethod("cbind2", c("DataTable", "DataTable"), function(x, y, ...) {
+  x <- as(x, "DataFrame")
+  y <- as(y, "DataFrame")
+  cbind(x, y, ...)
+})
+
+.mergeByHits <- function(x, y, by, all.x=FALSE, all.y=FALSE, sort = TRUE,
                          suffixes = c(".x", ".y"))
-{    
+{
     nm.x <- colnames(x)
     nm.y <- colnames(y)
     cnm <- nm.x %in% nm.y
-    if (any(cnm) && nzchar(suffixes[1L])) 
+    if (any(cnm) && nzchar(suffixes[1L]))
         nm.x[cnm] <- paste0(nm.x[cnm], suffixes[1L])
     cnm <- nm.y %in% nm.x
     if (any(cnm) && nzchar(suffixes[2L]))
@@ -196,7 +210,7 @@ setMethod("rbind2", c("DataTable", "DataTable"), function(x, y, ...) {
     }
     y <- y[c(to(by), if (all.x) rep.int(NA_integer_, length(x.alone)),
              if (all.y) y.alone), , drop = FALSE]
-    
+
     cbind(x, y)
 }
 
@@ -269,75 +283,83 @@ setMethod("as.data.frame", "DataTable",
                             row.names=row.names, optional=optional, ...)
           })
 
+
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### The "show" method.
+### Display
+###
+### Even though the "show" method and its helper functions below are defined
+### for DataTable derivatives, they're totally geared towards data-frame-like
+### objects. Don't get fooled!
+### TODO: Either make them work for DataTable derivatives **in general** or
+### move this to DataFrame-class.R
 ###
 
-setMethod("show", "DataTable",
-          function(object)
-          {
-              nhead <- get_showHeadLines()
-              ntail <- get_showTailLines()
-              nr <- nrow(object)
-              nc <- ncol(object)
-              cat(classNameForDisplay(object), " with ",
-                  nr, ifelse(nr == 1, " row and ", " rows and "),
-                  nc, ifelse(nc == 1, " column\n", " columns\n"),
-                  sep = "")
-              if (nr > 0 && nc > 0) {
-                  nms <- rownames(object)
-                  if (nr <= (nhead + ntail + 1L)) {
-                      out <-
-                        as.matrix(format(as.data.frame(
-                                         lapply(object, showAsCell),
-                                         optional = TRUE)))
-                      if (!is.null(nms))
-                          rownames(out) <- nms
-                  } else {
-                      out <-
-                        rbind(as.matrix(format(as.data.frame(
-                                               lapply(object, function(x)
-                                                      showAsCell(head(x, nhead))),
-                                               optional = TRUE))),
-                              rbind(rep.int("...", nc)),
-                              as.matrix(format(as.data.frame(
-                                               lapply(object, function(x) 
-                                                      showAsCell(tail(x, ntail))),
-                                               optional = TRUE))))
-                  rownames(out) <- .rownames(nms, nr, nhead, ntail) 
-                  }
-                  classinfo <-
-                    matrix(unlist(lapply(object, function(x) {
-                        paste0("<", classNameForDisplay(x)[1],
-                               ">")
-                    }), use.names = FALSE), nrow = 1,
-                           dimnames = list("", colnames(out)))
-                  out <- rbind(classinfo, out)
-                  print(out, quote = FALSE, right = TRUE)
-              }
-              invisible(NULL)
-          })
+setMethod("makeNakedCharacterMatrixForDisplay", "DataTable",
+    function(x)
+    {
+        df <- as.data.frame(lapply(x, showAsCell), optional=TRUE)
+        as.matrix(format(df))
+    }
+)
 
-.rownames <- function(nms, nrow, nhead, ntail)
+make_rownames_for_DataTable_display <- function(x_rownames, nrow, nhead, ntail)
 {
-    p1 <- ifelse (nhead == 0, 0L, 1L)
-    p2 <- ifelse (ntail == 0, 0L, ntail-1L)
+    p1 <- ifelse(nhead == 0L, 0L, 1L)
+    p2 <- ifelse(ntail == 0L, 0L, ntail - 1L)
     s1 <- s2 <- character(0)
-
-    if (is.null(nms)) {
-        if (nhead > 0) 
+    if (is.null(x_rownames)) {
+        if (nhead > 0L)
             s1 <- paste0(as.character(p1:nhead))
-        if (ntail > 0) 
+        if (ntail > 0L)
             s2 <- paste0(as.character((nrow-p2):nrow))
-    } else { 
-        if (nhead > 0) 
-            s1 <- paste0(head(nms, nhead))
-        if (ntail > 0) 
-            s2 <- paste0(tail(nms, ntail))
+    } else {
+        if (nhead > 0L)
+            s1 <- paste0(head(x_rownames, nhead))
+        if (ntail > 0L)
+            s2 <- paste0(tail(x_rownames, ntail))
     }
     c(s1, "...", s2)
 }
 
+make_class_info_for_DataTable_display <- function(x)
+{
+    vapply(x, function(xi) paste0("<", classNameForDisplay(xi), ">"),
+           character(1), USE.NAMES=FALSE)
+}
+
+.show_DataTable <- function(x)
+{
+    nhead <- get_showHeadLines()
+    ntail <- get_showTailLines()
+    x_nrow <- nrow(x)
+    x_ncol <- ncol(x)
+    cat(classNameForDisplay(x), " with ",
+        x_nrow, " row", ifelse(x_nrow == 1L, "", "s"),
+        " and ",
+        x_ncol, " column", ifelse(x_ncol == 1L, "", "s"),
+        "\n", sep="")
+    if (x_nrow != 0L && x_ncol != 0L) {
+        x_rownames <- rownames(x)
+        if (x_nrow <= nhead + ntail + 1L) {
+            m <- makeNakedCharacterMatrixForDisplay(x)
+            if (!is.null(x_rownames))
+                rownames(m) <- x_rownames
+        } else {
+            m <- rbind(makeNakedCharacterMatrixForDisplay(head(x, nhead)),
+                       rbind(rep.int("...", x_ncol)),
+                       makeNakedCharacterMatrixForDisplay(tail(x, ntail)))
+            rownames(m) <- make_rownames_for_DataTable_display(
+                                                       x_rownames, x_nrow,
+                                                       nhead, ntail)
+        }
+        m <- rbind(make_class_info_for_DataTable_display(x), m)
+        print(m, quote=FALSE, right=TRUE)
+    }
+    invisible(NULL)
+}
+
+setMethod("show", "DataTable", function(object) .show_DataTable(object))
+
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 ### Statistical routines
