@@ -8,7 +8,7 @@ setClass("FilterRules", representation(active = "logical"),
          prototype(elementType = "expression_OR_function"),
          contains = "SimpleList")
 
-setMethod("parallelSlotNames", "FilterRules",
+setMethod("vertical_slot_names", "FilterRules",
           function(x) c("active", callNextMethod()))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -372,7 +372,7 @@ FilterMatrix <- function(matrix, filterRules) {
 
 setMethod("show", "FilterMatrix", function(object) {
   cat(class(object), " (", nrow(object), " x ", ncol(object), ")\n", sep = "")
-  mat <- makePrettyMatrixForCompactPrinting(object, function(x) x@.Data)
+  mat <- makePrettyMatrixForCompactPrinting(object)
   print(mat, quote = FALSE, right = TRUE)
 })
 
