@@ -36,12 +36,13 @@ setClass("SortedByQuerySelfHits", contains=c("SelfHits", "SortedByQueryHits"))
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### parallelSlotNames()
+### vertical_slot_names()
 ###
 
-### Combine the new parallel slots with those of the parent class. Make sure
-### to put the new parallel slots *first*.
-setMethod("parallelSlotNames", "Hits",
+### Combine the new "vertical slots" with those of the parent class. Make
+### sure to put the new vertical slots **first**. See bindROWS.R file for
+### what slots should or should not be considered "vertical".
+setMethod("vertical_slot_names", "Hits",
     function(x) c("from", "to", callNextMethod())
 )
 
@@ -390,19 +391,15 @@ setMethod("summary", "Hits", summary.Hits)
 
 .from_Hits_to_naked_character_matrix_for_display <- function(x)
 {
-    x_len <- length(x)
-    x_mcols <- mcols(x, use.names=FALSE)
-    x_nmc <- if (is.null(x_mcols)) 0L else ncol(x_mcols)
-    ans <- cbind(from=as.character(from(x)),
-                 to=as.character(to(x)))
+    m <- cbind(from=as.character(from(x)),
+               to=as.character(to(x)))
     if (is(x, "SortedByQueryHits"))
-        colnames(ans) <- c("queryHits", "subjectHits")
-    if (x_nmc > 0L) {
-        tmp <- as.data.frame(lapply(x_mcols, showAsCell), optional=TRUE)
-        ans <- cbind(ans, `|`=rep.int("|", x_len), as.matrix(tmp))
-    }
-    ans
+        colnames(m) <- c("queryHits", "subjectHits")
+    cbind_mcols_for_display(m, x)
 }
+setMethod("makeNakedCharacterMatrixForDisplay", "Hits",
+    .from_Hits_to_naked_character_matrix_for_display
+)
 
 .show_Hits <- function(x, margin="", print.classinfo=FALSE,
                                      print.nnode=FALSE)
@@ -410,8 +407,7 @@ setMethod("summary", "Hits", summary.Hits)
     cat(margin, summary(x), ":\n", sep="")
     ## makePrettyMatrixForCompactPrinting() assumes that head() and tail()
     ## work on 'x'.
-    out <- makePrettyMatrixForCompactPrinting(x,
-                .from_Hits_to_naked_character_matrix_for_display)
+    out <- makePrettyMatrixForCompactPrinting(x)
     if (print.classinfo) {
         .COL2CLASS <- c(
             from="integer",
