@@ -14,7 +14,7 @@ setClass("Pairs",
                    second=logical(0L),
                    elementMetadata=DataFrame()))
 
-setMethod("parallelSlotNames", "Pairs", function(x)
+setMethod("vertical_slot_names", "Pairs", function(x)
     c("first", "second", "NAMES", callNextMethod()))
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
@@ -64,7 +64,7 @@ Pairs <- function(first, second, ..., names = NULL, hits = NULL) {
     if (!missing(...)) {
         elementMetadata <- DataFrame(...)
     } else {
-        elementMetadata <- make_zero_col_DataFrame(NROW(first))
+        elementMetadata <- make_zero_col_DFrame(NROW(first))
     }
     new("Pairs", first=first, second=second, NAMES=names,
                  elementMetadata=elementMetadata)
@@ -185,21 +185,15 @@ setMethod("t", "Pairs", function(x) {
 ### Show
 ###
 
-.makeNakedMatFromPairs <- function(x) {
-    x_len <- length(x)
-    x_mcols <- mcols(x, use.names=FALSE)
-    x_nmc <- if (is.null(x_mcols)) 
-                 0L
-             else ncol(x_mcols)
-    ans <- cbind(first = showAsCell(first(x)),
-                 second = showAsCell(second(x)))
-    if (x_nmc > 0L) {
-        tmp <- do.call(data.frame, c(lapply(x_mcols, showAsCell), 
-                                     list(check.names = FALSE)))
-        ans <- cbind(ans, `|` = rep.int("|", x_len), as.matrix(tmp))
-    }
-    ans
+.from_Pairs_to_naked_character_matrix_for_display <- function(x)
+{
+    m <- cbind(first=showAsCell(first(x)),
+               second=showAsCell(second(x)))
+    cbind_mcols_for_display(m, x)
 }
+setMethod("makeNakedCharacterMatrixForDisplay", "Pairs",
+    .from_Pairs_to_naked_character_matrix_for_display
+)
 
 showPairs <- function(x, margin = "", print.classinfo = FALSE) {
     x_class <- class(x)
@@ -211,7 +205,7 @@ showPairs <- function(x, margin = "", print.classinfo = FALSE) {
     cat(x_class, " object with ", x_len, " pair",
         ifelse(x_len ==  1L, "", "s"), " and ", x_nmc, " metadata column",
         ifelse(x_nmc == 1L, "", "s"), ":\n", sep = "")
-    out <- makePrettyMatrixForCompactPrinting(x, .makeNakedMatFromPairs)
+    out <- makePrettyMatrixForCompactPrinting(x)
     if (print.classinfo) {
         .COL2CLASS <- c(first = class(first(x)), second = class(second(x)))
         classinfo <- makeClassinfoRowForCompactPrinting(x, .COL2CLASS)
