@@ -12,12 +12,13 @@ setClass("SimpleList",
 
 
 ### - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
-### parallelSlotNames()
+### vertical_slot_names()
 ###
 
-### Combine the new parallel slots with those of the parent class. Make sure
-### to put the new parallel slots *first*.
-setMethod("parallelSlotNames", "SimpleList",
+### Combine the new "vertical slots" with those of the parent class. Make
+### sure to put the new vertical slots **first**. See bindROWS.R file for
+### what slots should or should not be considered "vertical".
+setMethod("vertical_slot_names", "SimpleList",
     function(x) c("listData", callNextMethod())
 )
 
@@ -60,7 +61,7 @@ new_SimpleList_from_list <- function(Class, x, ..., mcols)
     proto <- new(Class)
     ans_elementType <- elementType(proto)
     if (is(S4Vectors::mcols(proto, use.names=FALSE), "DataFrame"))
-        mcols <- make_zero_col_DataFrame(length(x))
+        mcols <- make_zero_col_DFrame(length(x))
     if (!all(sapply(x, function(xi) extends(class(xi), ans_elementType))))
         stop("all elements in 'x' must be ", ans_elementType, " objects")
     if (missing(mcols))
