@@ -119,6 +119,8 @@ void R_init_S4Vectors(DllInfo *info)
 	REGISTER_CCALLABLE(_safe_int_add);
 	REGISTER_CCALLABLE(_safe_int_mult);
 	REGISTER_CCALLABLE(_as_int);
+	REGISTER_CCALLABLE(_safe_llint_add);
+	REGISTER_CCALLABLE(_safe_llint_mult);
 
 /* sort_utils.c */
 	REGISTER_CCALLABLE(_sort_ints);
